@@ -38,6 +38,8 @@ our @EXPORT_OK = qw(
 
 our @EXPORT = qw(
 	SASS_STYLE_NESTED
+	SASS_STYLE_EXPANDED
+	SASS_STYLE_COMPACT
 	SASS_STYLE_COMPRESSED
 	SASS2SCSS_PRETTIFY_0
 	SASS2SCSS_PRETTIFY_1
@@ -48,63 +50,108 @@ our @EXPORT = qw(
 	SASS2SCSS_CONVERT_COMMENT
 );
 
-our $VERSION = "v3.1.1";
+our $VERSION = "v3.2.0";
 
 require XSLoader;
 XSLoader::load('CSS::Sass', $VERSION);
 require CSS::Sass::Type;
 
-sub new {
+sub new
+{
     my ($class, %options) = @_;
     # Ensure initial sub structures on options
+    $options{plugin_paths} = [] unless exists $options{plugin_paths};
     $options{include_paths} = [] unless exists $options{include_paths};
     $options{sass_functions} = {} unless exists $options{sass_functions};
     # Create and return new object with options
     bless { options => \%options }, $class;
 };
 
-sub options {
+sub options
+{
     shift->{options}
 }
 
-sub last_error {
+sub last_error
+{
     my ($self) = @_;
     $self->{last_error}
 }
 
-sub sass_compile {
+my @path_types = (
+  'plugin_paths',
+  'include_paths'
+);
+
+# directory delimiter according to platform
+my $dir_delim = $^O eq 'MSWin32' ? ';' : ':';
+
+# normalize option hash
+my $normalize_options = sub
+{
+    my ($options) = @_;
+    # gather all functions
+    # they need to be hashes
+    my %functions =
+    (
+      %{$options->{'functions'} || {}},
+      %{$options->{'sass_functions'} || {}}
+    );
+    # create functions array
+    # help the c code a little
+    my @functions = map { [
+      $_, $functions{$_}
+    ] } keys %functions;
+    # gather all importers
+    # they need to be arrays
+    my @importers =
+    map {
+      ref($_) eq "ARRAY" ?
+        $_ : [ $_, 0 ];
+    }
+    grep { defined }
+    (
+      $options->{'importer'},
+      @{$options->{'importers'} || []},
+      @{$options->{'sass_importers'} || []}
+    );
+    # gather all paths strings
+    foreach my $type (@path_types)
+    {
+      $options->{$type} = join $dir_delim,
+        map { split $dir_delim, $_ }
+        @{$options->{$type} || []};
+    }
+    # now normalize the original hash
+    $options->{'functions'} = \@functions;
+    $options->{'importers'} = \@importers;
+    # remove importer from options
+    # it is now included in importers
+    delete $options->{'importer'};
+    # return pointer
+    return $options;
+};
+
+sub sass_compile
+{
     my ($sass_code, %options) = @_;
     no warnings 'uninitialized';
-    my $r = compile_sass($sass_code, { %options,
-                                       # Override sass_functions with the arrayref of arrayrefs that the XS expects.
-                                       !$options{sass_functions} ? ()
-                                                                 : (sass_functions => [ map { [ $_ => $options{sass_functions}->{$_} ]
-                                                                                            } keys %{$options{sass_functions}} ]),
-                                       # Override include_paths with a ':' separated list
-                                       !$options{include_paths} ? ()
-                                                                : (include_paths => join($^O eq 'MSWin32' ? ';' : ':',
-                                                                                         @{$options{include_paths}})),
-                                     });
+    $normalize_options->(\%options);
+    my $r = compile_sass($sass_code, \%options);
     wantarray ? ($r->{output_string}, $r->{error_message}, $r) : $r->{output_string}
 }
 
-sub sass_compile_file {
+sub sass_compile_file
+{
     my ($input_path, %options) = @_;
     no warnings 'uninitialized';
-    my $r = compile_sass_file($input_path, { %options,
-                                            # Override sass_functions with the arrayref of arrayrefs that the XS expects.
-                                            !$options{sass_functions} ? ()
-                                                                      : (sass_functions => [ map { [ $_ => $options{sass_functions}->{$_} ]
-                                                                                                 } keys %{$options{sass_functions}} ]),
-                                            # Override include_paths with a ':' separated list
-                                            !$options{include_paths} ? ()
-                                                                     : (include_paths => join($^O eq 'MSWin32' ? ';' : ':',
-                                                                                              @{$options{include_paths}})),
-                                          });
+    $normalize_options->(\%options);
+    my $r = compile_sass_file($input_path, \%options);
     wantarray ? ($r->{output_string}, $r->{error_message}, $r) : $r->{output_string}
 }
 
-sub compile {
+sub compile
+{
     my ($self, $sass_code) = @_;
     my ($compiled, $stats);
     ($compiled, $self->{last_error}, $stats) = sass_compile($sass_code, %{$self->options});
@@ -112,7 +159,8 @@ sub compile {
     wantarray ? ($compiled, $stats) : $compiled
 }
 
-sub compile_file {
+sub compile_file
+{
     my ($self, $sass_file) = @_;
     my ($compiled, $stats);
     ($compiled, $self->{last_error}, $stats) = sass_compile_file($sass_file, %{$self->options});
@@ -149,8 +197,8 @@ CSS::Sass - Compile .scss files using libsass
 
 
   # Object Oriented API w/ options
-  my $sass = CSS::Sass->new(include_paths   => ['some/include/path'],
-                            image_path      => 'base_url',
+  my $sass = CSS::Sass->new(plugin_paths    => ['plugins'],
+                            include_paths   => ['some/include/path'],
                             output_style    => SASS_STYLE_COMPRESSED,
                             source_map_file => 'output.css.map',
                             source_comments => 1,
@@ -180,7 +228,6 @@ CSS::Sass - Compile .scss files using libsass
   # Functional API w/ options
   my ($css, $err, $srcmap) = sass_compile('A { color: red; }',
                                           include_paths   => ['some/include/path'],
-                                          image_path      => 'base_url',
                                           output_style    => SASS_STYLE_NESTED,
                                           source_map_file => 'output.css.map');
 
@@ -273,6 +320,18 @@ C<undef>, but never both.
 The default is C<SASS_STYLE_NESTED>. Set to C<SASS_STYLE_COMPRESSED> to
 eliminate all whitespace (for your production CSS).
 
+=item C<precision>
+
+Set the floating point precision for output.
+
+=item C<linefeed>
+
+Set the linefeed string used for css output.
+
+=item C<indentation>
+
+Set the indentation string used for css output.
+
 =item C<source_comments>
 
 Set to C<true> to get extra comments in the output, indicating what input
@@ -284,6 +343,10 @@ Setting this option enables the source-map generating. The file will not
 actually be created, but its content will be returned to the caller. It
 will also enable sourceMappingURL comment by default. See C<no_src_map_url>.
 
+=item C<source_map_root>
+
+A path (string) that is directly embedded in the source map as C<sourceRoot>.
+
 =item C<source_map_embed>
 
 Embeds the complete source-map content into the sourceMappingURL, by using
@@ -305,17 +368,11 @@ Setting this options makes C<source_map_embed> useless.
 This is an arrayref that holds the list a of paths to search (in addition to
 the current directory) when following Sass C<@import> directives.
 
-=item C<image_path>
+=item C<plugin_paths>
 
-This is a string that holds the base URL. This is only used in the
-(non-standard) C<image-url()> Sass function. For example, if C<image_path>
-is set to C<'file:///tmp/a/b/c'>, then the follwoing Sass code:
-
-  .something { background-image: image-url("my/path"); }
-
-...will compile to this:
-
-  .something { background-image: url("file:///tmp/a/b/c/my/path"); }
+This is an arrayref that holds a list of paths to search for third-party
+plugins. It will automatically load any <dll> or <so> library within that
+directory. This is currently a highly experimental libsass feature!
 
 =item C<dont_die>
 
@@ -474,7 +531,7 @@ L<The CSS::Sass Home Page|https://github.com/sass/perl-libsass>
 
 =head1 AUTHOR
 
-David Caldwell E<lt>david@porkrind.orgE<gt>  
+David Caldwell E<lt>david@porkrind.orgE<gt>
 Marcel Greter E<lt>perl-libsass@ocbnet.chE<gt>
 
 =head1 LICENSE
