@@ -8,6 +8,8 @@
 namespace Sass {
   using namespace std;
 
+  static Null sass_null(Sass::Null(ParserState("null")));
+
   bool Compound_Selector::operator<(const Compound_Selector& rhs) const
   {
     To_String to_string;
@@ -89,7 +91,7 @@ namespace Sass {
 
   Compound_Selector* Simple_Selector::unify_with(Compound_Selector* rhs, Context& ctx)
   {
-    To_String to_string;
+    To_String to_string(&ctx);
     for (size_t i = 0, L = rhs->length(); i < L; ++i)
     { if (perform(&to_string) == (*rhs)[i]->perform(&to_string)) return rhs; }
 
@@ -183,6 +185,7 @@ namespace Sass {
           return 0;
       }
     }
+    rhs->has_line_break(has_line_break());
     return Simple_Selector::unify_with(rhs, ctx);
   }
 
@@ -413,7 +416,10 @@ namespace Sass {
   {
     if (!tail()) return 0;
     if (!head()) return tail()->context(ctx);
-    return new (ctx.mem) Complex_Selector(pstate(), combinator(), head(), tail()->context(ctx));
+    Complex_Selector* cpy = new (ctx.mem) Complex_Selector(pstate(), combinator(), head(), tail()->context(ctx));
+    cpy->media_block(media_block());
+    cpy->last_block(last_block());
+    return cpy;
   }
 
   Complex_Selector* Complex_Selector::innermost()
@@ -451,7 +457,7 @@ namespace Sass {
   {
     Complex_Selector* cpy = new (ctx.mem) Complex_Selector(*this);
 
-		if (head()) {
+    if (head()) {
     	cpy->head(head()->clone(ctx));
     }
 
@@ -470,10 +476,11 @@ namespace Sass {
 
 
 
+  /* not used anymore - remove?
   Selector_Placeholder* Selector::find_placeholder()
   {
     return 0;
-  }
+  }*/
 
   void Selector_List::adjust_after_pushing(Complex_Selector* c)
   {
@@ -486,6 +493,7 @@ namespace Sass {
 #endif
   }
 
+  /* not used anymore - remove?
   Selector_Placeholder* Selector_List::find_placeholder()
   {
     if (has_placeholder()) {
@@ -494,8 +502,9 @@ namespace Sass {
       }
     }
     return 0;
-  }
+  }*/
 
+  /* not used anymore - remove?
   Selector_Placeholder* Complex_Selector::find_placeholder()
   {
     if (has_placeholder()) {
@@ -503,8 +512,9 @@ namespace Sass {
       else if (tail() && tail()->has_placeholder()) return tail()->find_placeholder();
     }
     return 0;
-  }
+  }*/
 
+  /* not used anymore - remove?
   Selector_Placeholder* Compound_Selector::find_placeholder()
   {
     if (has_placeholder()) {
@@ -514,12 +524,13 @@ namespace Sass {
       // return this;
     }
     return 0;
-  }
+  }*/
 
+  /* not used anymore - remove?
   Selector_Placeholder* Selector_Placeholder::find_placeholder()
   {
     return this;
-  }
+  }*/
 
   vector<string> Compound_Selector::to_str_vec()
   {
@@ -533,7 +544,7 @@ namespace Sass {
 
   Compound_Selector* Compound_Selector::minus(Compound_Selector* rhs, Context& ctx)
   {
-    To_String to_string;
+    To_String to_string(&ctx);
     Compound_Selector* result = new (ctx.mem) Compound_Selector(pstate());
 
     // not very efficient because it needs to preserve order
@@ -562,6 +573,7 @@ namespace Sass {
     }
   }
 
+  /* not used anymore - remove?
   vector<Compound_Selector*> Complex_Selector::to_vector()
   {
     vector<Compound_Selector*> result;
@@ -575,6 +587,13 @@ namespace Sass {
       if (h) result.push_back(h);
     }
     return result;
+  }*/
+
+  Expression* Hashed::at(Expression* k) const
+  {
+    if (elements_.count(k))
+    { return elements_.at(k); }
+    else { return &sass_null; }
   }
 
 }
