@@ -5,34 +5,41 @@
 #endif
 
 #include "ast.hpp"
+#include "util.hpp"
+#include "sass.h"
 #include "context.hpp"
+#include "plugins.hpp"
 #include "constants.hpp"
 #include "parser.hpp"
 #include "file.hpp"
 #include "inspect.hpp"
-#include "output_nested.hpp"
-#include "output_compressed.hpp"
+#include "output.hpp"
 #include "expand.hpp"
 #include "eval.hpp"
 #include "contextualize.hpp"
+#include "contextualize_eval.hpp"
 #include "cssize.hpp"
+#include "listize.hpp"
 #include "extend.hpp"
 #include "remove_placeholders.hpp"
-#include "copy_c_str.hpp"
 #include "color_names.hpp"
 #include "functions.hpp"
 #include "backtrace.hpp"
 #include "sass2scss.h"
 #include "prelexer.hpp"
+#include "emitter.hpp"
 
-#include <iomanip>
-#include <iostream>
+#include <string>
+#include <cstdlib>
 #include <cstring>
+#include <iomanip>
 #include <sstream>
+#include <iostream>
 
 namespace Sass {
   using namespace Constants;
   using namespace File;
+  using namespace Sass;
   using std::cerr;
   using std::endl;
 
@@ -43,35 +50,44 @@ namespace Sass {
     this->source = source;
   }
 
+  inline bool sort_importers (const Sass_Importer_Entry& i, const Sass_Importer_Entry& j)
+  { return sass_importer_get_priority(i) > sass_importer_get_priority(j); }
 
   Context::Context(Context::Data initializers)
-  : mem(Memory_Manager<AST_Node>()),
+  : // Output(this),
+    head_imports(0),
+    mem(Memory_Manager<AST_Node>()),
+    c_options               (initializers.c_options()),
+    c_compiler              (initializers.c_compiler()),
     source_c_str            (initializers.source_c_str()),
     sources                 (vector<const char*>()),
+    plugin_paths            (initializers.plugin_paths()),
     include_paths           (initializers.include_paths()),
     queue                   (vector<Sass_Queued>()),
     style_sheets            (map<string, Block*>()),
-    source_map              (resolve_relative_path(initializers.output_path(), initializers.source_map_file(), get_cwd())),
-    c_functions             (vector<Sass_C_Function_Callback>()),
+    emitter (this),
+    c_headers               (vector<Sass_Importer_Entry>()),
+    c_importers             (vector<Sass_Importer_Entry>()),
+    c_functions             (vector<Sass_Function_Entry>()),
     indent                  (initializers.indent()),
     linefeed                (initializers.linefeed()),
-    image_path              (initializers.image_path()),
     input_path              (make_canonical_path(initializers.input_path())),
     output_path             (make_canonical_path(initializers.output_path())),
     source_comments         (initializers.source_comments()),
     output_style            (initializers.output_style()),
     source_map_file         (make_canonical_path(initializers.source_map_file())),
+    source_map_root         (initializers.source_map_root()), // pass-through
     source_map_embed        (initializers.source_map_embed()),
     source_map_contents     (initializers.source_map_contents()),
     omit_source_map_url     (initializers.omit_source_map_url()),
     is_indented_syntax_src  (initializers.is_indented_syntax_src()),
-    importer                (initializers.importer()),
     names_to_colors         (map<string, Color*>()),
     colors_to_names         (map<int, string>()),
     precision               (initializers.precision()),
-    _skip_source_map_update (initializers._skip_source_map_update()),
+    plugins(),
     subset_map              (Subset_Map<string, pair<Complex_Selector*, Compound_Selector*> >())
   {
+
     cwd = get_cwd();
 
     // enforce some safe defaults
@@ -81,10 +97,28 @@ namespace Sass {
 
     include_paths.push_back(cwd);
     collect_include_paths(initializers.include_paths_c_str());
-    collect_include_paths(initializers.include_paths_array());
+    // collect_include_paths(initializers.include_paths_array());
+    collect_plugin_paths(initializers.plugin_paths_c_str());
+    // collect_plugin_paths(initializers.plugin_paths_array());
 
     setup_color_map();
 
+    for (size_t i = 0, S = plugin_paths.size(); i < S; ++i) {
+      plugins.load_plugins(plugin_paths[i]);
+    }
+
+    for(auto fn : plugins.get_functions()) {
+      c_functions.push_back(fn);
+    }
+    for(auto fn : plugins.get_headers()) {
+      c_headers.push_back(fn);
+    }
+    for(auto fn : plugins.get_importers()) {
+      c_importers.push_back(fn);
+    }
+
+    sort (c_headers.begin(), c_headers.end(), sort_importers);
+    sort (c_importers.begin(), c_importers.end(), sort_importers);
     string entry_point = initializers.entry_point();
     if (!entry_point.empty()) {
       string result(add_file(entry_point));
@@ -92,13 +126,35 @@ namespace Sass {
         throw "File to read not found or unreadable: " + entry_point;
       }
     }
+
+    emitter.set_filename(resolve_relative_path(output_path, source_map_file, cwd));
+
+  }
+
+  void Context::add_c_function(Sass_Function_Entry function)
+  {
+    c_functions.push_back(function);
+  }
+  void Context::add_c_header(Sass_Importer_Entry header)
+  {
+    c_headers.push_back(header);
+    // need to sort the array afterwards (no big deal)
+    sort (c_headers.begin(), c_headers.end(), sort_importers);
+  }
+  void Context::add_c_importer(Sass_Importer_Entry importer)
+  {
+    c_importers.push_back(importer);
+    // need to sort the array afterwards (no big deal)
+    sort (c_importers.begin(), c_importers.end(), sort_importers);
   }
 
   Context::~Context()
   {
     // everything that gets put into sources will be freed by us
-    for (size_t i = 0; i < sources.size(); ++i) delete[] sources[i];
     for (size_t n = 0; n < import_stack.size(); ++n) sass_delete_import(import_stack[n]);
+    // sources are allocated by strdup or malloc (overtaken from C code)
+    for (size_t i = 0; i < sources.size(); ++i) free((void*)sources[i]);
+    // clear inner structures (vectors)
     sources.clear(); import_stack.clear();
   }
 
@@ -115,9 +171,9 @@ namespace Sass {
       names_to_colors[name] = value;
       // only map fully opaque colors
       if (color_values[i*4+3] >= 1) {
-        int numval = color_values[i*4]*0x10000;
-        numval += color_values[i*4+1]*0x100;
-        numval += color_values[i*4+2];
+        int numval = static_cast<int>(color_values[i*4])*0x10000;
+        numval += static_cast<int>(color_values[i*4+1])*0x100;
+        numval += static_cast<int>(color_values[i*4+2]);
         colors_to_names[numval] = name;
       }
       ++i;
@@ -151,7 +207,6 @@ namespace Sass {
 
   void Context::collect_include_paths(const char** paths_array)
   {
-    if (*include_paths.back().rbegin() != '/') include_paths.back() += '/';
     if (paths_array) {
       for (size_t i = 0; paths_array[i]; i++) {
         collect_include_paths(paths_array[i]);
@@ -159,108 +214,112 @@ namespace Sass {
     }
   }
 
+  void Context::collect_plugin_paths(const char* paths_str)
+  {
+
+    if (paths_str) {
+      const char* beg = paths_str;
+      const char* end = Prelexer::find_first<PATH_SEP>(beg);
+
+      while (end) {
+        string path(beg, end - beg);
+        if (!path.empty()) {
+          if (*path.rbegin() != '/') path += '/';
+          plugin_paths.push_back(path);
+        }
+        beg = end + 1;
+        end = Prelexer::find_first<PATH_SEP>(beg);
+      }
+
+      string path(beg);
+      if (!path.empty()) {
+        if (*path.rbegin() != '/') path += '/';
+        plugin_paths.push_back(path);
+      }
+    }
+  }
+
+  void Context::collect_plugin_paths(const char** paths_array)
+  {
+    if (paths_array) {
+      for (size_t i = 0; paths_array[i]; i++) {
+        collect_plugin_paths(paths_array[i]);
+      }
+    }
+  }
   void Context::add_source(string load_path, string abs_path, const char* contents)
   {
     sources.push_back(contents);
     included_files.push_back(abs_path);
     queue.push_back(Sass_Queued(load_path, abs_path, contents));
-    source_map.source_index.push_back(sources.size() - 1);
+    emitter.add_source_index(sources.size() - 1);
     include_links.push_back(resolve_relative_path(abs_path, source_map_file, cwd));
   }
 
-  string Context::add_file(string path)
+  // Add a new import file to the context
+  string Context::add_file(const string& file)
   {
     using namespace File;
-    char* contents = 0;
-    string real_path;
-    path = make_canonical_path(path);
-    for (size_t i = 0, S = include_paths.size(); i < S; ++i) {
-      string full_path(join_paths(include_paths[i], path));
-      if (style_sheets.count(full_path)) return full_path;
-      contents = resolve_and_load(full_path, real_path);
-      if (contents) {
-        add_source(full_path, real_path, contents);
-        style_sheets[full_path] = 0;
-        return full_path;
-      }
+    string path(make_canonical_path(file));
+    string resolved(find_file(path, include_paths));
+    if (resolved == "") return resolved;
+    if (char* contents = read_file(resolved)) {
+      add_source(path, resolved, contents);
+      style_sheets[path] = 0;
+      return path;
     }
-    return string();
+    return string("");
   }
 
-  string Context::add_file(string dir, string rel_filepath)
+  // Add a new import file to the context
+  // This has some previous directory context
+  string Context::add_file(const string& base, const string& file)
   {
     using namespace File;
-    char* contents = 0;
-    string real_path;
-    rel_filepath = make_canonical_path(rel_filepath);
-    string full_path(join_paths(dir, rel_filepath));
-    if (style_sheets.count(full_path)) return full_path;
-    contents = resolve_and_load(full_path, real_path);
-    if (contents) {
-      add_source(full_path, real_path, contents);
-      style_sheets[full_path] = 0;
-      return full_path;
+    string path(make_canonical_path(file));
+    string base_file(join_paths(base, path));
+    string resolved(resolve_file(base_file));
+    if (style_sheets.count(base_file)) return base_file;
+    if (char* contents = read_file(resolved)) {
+      add_source(base_file, resolved, contents);
+      style_sheets[base_file] = 0;
+      return base_file;
     }
-    for (size_t i = 0, S = include_paths.size(); i < S; ++i) {
-      string full_path(join_paths(include_paths[i], rel_filepath));
-      if (style_sheets.count(full_path)) return full_path;
-      contents = resolve_and_load(full_path, real_path);
-      if (contents) {
-        add_source(full_path, real_path, contents);
-        style_sheets[full_path] = 0;
-        return full_path;
-      }
-    }
-    return string();
+    // now go the regular code path
+    return add_file(path);
   }
 
   void register_function(Context&, Signature sig, Native_Function f, Env* env);
   void register_function(Context&, Signature sig, Native_Function f, size_t arity, Env* env);
   void register_overload_stub(Context&, string name, Env* env);
   void register_built_in_functions(Context&, Env* env);
-  void register_c_functions(Context&, Env* env, Sass_C_Function_List);
-  void register_c_function(Context&, Env* env, Sass_C_Function_Callback);
+  void register_c_functions(Context&, Env* env, Sass_Function_List);
+  void register_c_function(Context&, Env* env, Sass_Function_Entry);
 
   char* Context::compile_block(Block* root)
   {
-    char* result = 0;
     if (!root) return 0;
-    switch (output_style) {
-      case COMPRESSED: {
-        Output_Compressed output_compressed(this);
-        root->perform(&output_compressed);
-        string output = output_compressed.get_buffer();
-        if (source_map_file != "" && !omit_source_map_url) {
-          output += format_source_mapping_url(source_map_file);
-        }
-        result = copy_c_str(output.c_str());
-      } break;
-
-      default: {
-        Output_Nested output_nested(source_comments, this);
-        root->perform(&output_nested);
-        string output = output_nested.get_buffer();
-        if (source_map_file != "" && !omit_source_map_url) {
-          output += linefeed + format_source_mapping_url(source_map_file);
-        }
-        result = copy_c_str(output.c_str());
-
-      } break;
+    root->perform(&emitter);
+    emitter.finalize();
+    OutputBuffer emitted = emitter.get_buffer();
+    string output = emitted.buffer;
+    if (source_map_file != "" && !omit_source_map_url) {
+      output += linefeed + format_source_mapping_url(source_map_file);
     }
-    return result;
+    return sass_strdup(output.c_str());
   }
 
   Block* Context::parse_file()
   {
     Block* root = 0;
     for (size_t i = 0; i < queue.size(); ++i) {
-      struct Sass_Import* import = sass_make_import(
+      Sass_Import_Entry import = sass_make_import(
         queue[i].load_path.c_str(),
         queue[i].abs_path.c_str(),
         0, 0
       );
       import_stack.push_back(import);
-      Parser p(Parser::from_c_str(queue[i].source, *this, ParserState(queue[i].abs_path, i)));
+      Parser p(Parser::from_c_str(queue[i].source, *this, ParserState(queue[i].abs_path, queue[i].source, i)));
       Block* ast = p.parse();
       sass_delete_import(import_stack.back());
       import_stack.pop_back();
@@ -274,13 +333,12 @@ namespace Sass {
     for (size_t i = 0, S = c_functions.size(); i < S; ++i) {
       register_c_function(*this, &tge, c_functions[i]);
     }
-    Eval eval(*this, &tge, &backtrace);
-    Contextualize contextualize(*this, &eval, &tge, &backtrace);
-    Expand expand(*this, &eval, &contextualize, &tge, &backtrace);
+    Contextualize contextualize(*this, &tge, &backtrace);
+    Listize listize(*this);
+    Eval eval(*this, &contextualize, &listize, &tge, &backtrace);
+    Contextualize_Eval contextualize_eval(*this, &eval, &tge, &backtrace);
+    Expand expand(*this, &eval, &contextualize_eval, &tge, &backtrace);
     Cssize cssize(*this, &tge, &backtrace);
-    // Inspect inspect(this);
-    // Output_Nested output_nested(*this);
-
     root = root->perform(&expand)->block();
     root = root->perform(&cssize)->block();
     if (!subset_map.empty()) {
@@ -299,11 +357,12 @@ namespace Sass {
     if (!source_c_str) return 0;
     queue.clear();
     if(is_indented_syntax_src) {
-      char * contents = sass2scss(source_c_str, SASS2SCSS_PRETTIFY_1);
+      char * contents = sass2scss(source_c_str, SASS2SCSS_PRETTIFY_1 | SASS2SCSS_KEEP_COMMENT);
       add_source(input_path, input_path, contents);
+      delete [] source_c_str;
       return parse_file();
     }
-    add_source(input_path, input_path, strdup(source_c_str));
+    add_source(input_path, input_path, source_c_str);
     return parse_file();
   }
 
@@ -323,7 +382,7 @@ namespace Sass {
   {
     string url = resolve_relative_path(file, output_path, cwd);
     if (source_map_embed) {
-      string map = source_map.generate_source_map(*this);
+      string map = emitter.generate_source_map(*this);
       istringstream is( map );
       ostringstream buffer;
       base64::encoder E;
@@ -338,8 +397,8 @@ namespace Sass {
   {
     if (source_map_file == "") return 0;
     char* result = 0;
-    string map = source_map.generate_source_map(*this);
-    result = copy_c_str(map.c_str());
+    string map = emitter.generate_source_map(*this);
+    result = sass_strdup(map.c_str());
     return result;
   }
 
@@ -347,7 +406,9 @@ namespace Sass {
   std::vector<std::string> Context::get_included_files(size_t skip)
   {
       vector<string> includes = included_files;
+      if (includes.size() == 0) return includes;
       std::sort( includes.begin() + skip, includes.end() );
+      includes.erase( includes.begin(), includes.begin() + skip );
       includes.erase( std::unique( includes.begin(), includes.end() ), includes.end() );
       // the skip solution seems more robust, as we may have real files named stdin
       // includes.erase( std::remove( includes.begin(), includes.end(), "stdin" ), includes.end() );
@@ -382,6 +443,7 @@ namespace Sass {
                                             name,
                                             0,
                                             0,
+                                            &ctx,
                                             true);
     (*env)[name + "[f]"] = stub;
   }
@@ -475,28 +537,21 @@ namespace Sass {
     // Boolean Functions
     register_function(ctx, not_sig, sass_not, env);
     register_function(ctx, if_sig, sass_if, env);
-    // Path Functions
-    register_function(ctx, image_url_sig, image_url, env);
     // Misc Functions
     register_function(ctx, inspect_sig, inspect, env);
     register_function(ctx, unique_id_sig, unique_id, env);
   }
 
-  void register_c_functions(Context& ctx, Env* env, Sass_C_Function_List descrs)
+  void register_c_functions(Context& ctx, Env* env, Sass_Function_List descrs)
   {
     while (descrs && *descrs) {
       register_c_function(ctx, env, *descrs);
       ++descrs;
     }
   }
-  void register_c_function(Context& ctx, Env* env, Sass_C_Function_Callback descr)
+  void register_c_function(Context& ctx, Env* env, Sass_Function_Entry descr)
   {
-    Definition* def = make_c_function(
-      sass_function_get_signature(descr),
-      sass_function_get_function(descr),
-      sass_function_get_cookie(descr),
-      ctx
-    );
+    Definition* def = make_c_function(descr, ctx);
     def->environment(env);
     (*env)[def->name() + "[f]"] = def;
   }
