@@ -9,12 +9,12 @@ namespace Sass {
   { }
 
   void error(string msg, ParserState pstate)
-  { throw Sass_Error(Sass_Error::syntax, pstate, msg); }
+  {
+    throw Sass_Error(Sass_Error::syntax, pstate, msg);
+  }
 
   void error(string msg, ParserState pstate, Backtrace* bt)
   {
-    if (!pstate.path.empty() && Prelexer::quoted_string(pstate.path.c_str()))
-      pstate.path = pstate.path.substr(1, pstate.path.size() - 1);
 
     Backtrace top(bt, pstate, "");
     msg += top.to_string();
