@@ -31,12 +31,21 @@ namespace Sass {
     add, sub, mul, div, fmod
   };
 
-  Eval::Eval(Context& ctx, Env* env, Backtrace* bt)
-  : ctx(ctx), env(env), backtrace(bt) { }
+  Eval::Eval(Context& ctx, Contextualize* contextualize, Listize* listize, Env* env, Backtrace* bt)
+  : ctx(ctx), contextualize(contextualize), listize(listize), env(env), backtrace(bt) { }
   Eval::~Eval() { }
 
   Eval* Eval::with(Env* e, Backtrace* bt) // for setting the env before eval'ing an expression
   {
+    contextualize = contextualize->with(0, e, bt);
+    env = e;
+    backtrace = bt;
+    return this;
+  }
+
+  Eval* Eval::with(Selector* c, Env* e, Backtrace* bt, Selector* p, Selector* ex) // for setting the env before eval'ing an expression
+  {
+    contextualize = contextualize->with(c, e, bt, p, ex);
     env = e;
     backtrace = bt;
     return this;
@@ -55,12 +64,59 @@ namespace Sass {
   Expression* Eval::operator()(Assignment* a)
   {
     string var(a->variable());
-    if (env->has(var)) {
-      Expression* v = static_cast<Expression*>((*env)[var]);
-      if (!a->is_guarded() || v->concrete_type() == Expression::NULL_VAL) (*env)[var] = a->value()->perform(this);
+    if (a->is_global()) {
+      if (a->is_default()) {
+        if (env->has_global(var)) {
+          Expression* e = dynamic_cast<Expression*>(env->get_global(var));
+          if (!e || e->concrete_type() == Expression::NULL_VAL) {
+            env->set_global(var, a->value()->perform(this));
+          }
+        }
+        else {
+          env->set_global(var, a->value()->perform(this));
+        }
+      }
+      else {
+        env->set_global(var, a->value()->perform(this));
+      }
+    }
+    else if (a->is_default()) {
+      if (env->has_lexical(var)) {
+        auto cur = env;
+        while (cur && cur->is_lexical()) {
+          if (cur->has_local(var)) {
+            if (AST_Node* node = cur->get_local(var)) {
+              Expression* e = dynamic_cast<Expression*>(node);
+              if (!e || e->concrete_type() == Expression::NULL_VAL) {
+                cur->set_local(var, a->value()->perform(this));
+              }
+            }
+            else {
+              throw runtime_error("Env not in sync");
+            }
+            return 0;
+          }
+          cur = cur->parent();
+        }
+        throw runtime_error("Env not in sync");
+      }
+      else if (env->has_global(var)) {
+        if (AST_Node* node = env->get_global(var)) {
+          Expression* e = dynamic_cast<Expression*>(node);
+          if (!e || e->concrete_type() == Expression::NULL_VAL) {
+            env->set_global(var, a->value()->perform(this));
+          }
+        }
+      }
+      else if (env->is_lexical()) {
+        env->set_local(var, a->value()->perform(this));
+      }
+      else {
+        env->set_local(var, a->value()->perform(this));
+      }
     }
     else {
-      env->current_frame()[var] = a->value()->perform(this);
+      env->set_lexical(var, a->value()->perform(this));
     }
     return 0;
   }
@@ -77,6 +133,8 @@ namespace Sass {
     return 0;
   }
 
+  // For does not create a new env scope
+  // But iteration vars are reset afterwards
   Expression* Eval::operator()(For* f)
   {
     string variable(f->variable());
@@ -88,19 +146,30 @@ namespace Sass {
     if (high->concrete_type() != Expression::NUMBER) {
       error("upper bound of `@for` directive must be numeric", high->pstate());
     }
-    double start = static_cast<Number*>(low)->value();
-    double end = static_cast<Number*>(high)->value();
-    Env new_env;
-    new_env[variable] = new (ctx.mem) Number(low->pstate(), start);
-    new_env.link(env);
-    env = &new_env;
+    Number* sass_start = static_cast<Number*>(low);
+    Number* sass_end = static_cast<Number*>(high);
+    // check if units are valid for sequence
+    if (sass_start->unit() != sass_end->unit()) {
+      stringstream msg; msg << "Incompatible units: '"
+        << sass_start->unit() << "' and '"
+        << sass_end->unit() << "'.";
+      error(msg.str(), low->pstate(), backtrace);
+    }
+    double start = sass_start->value();
+    double end = sass_end->value();
+    // only create iterator once in this environment
+    Number* it = new (env->mem) Number(low->pstate(), start, sass_end->unit());
+    AST_Node* old_var = env->has_local(variable) ? env->get_local(variable) : 0;
+    env->set_local(variable, it);
     Block* body = f->block();
     Expression* val = 0;
     if (start < end) {
       if (f->is_inclusive()) ++end;
       for (double i = start;
            i < end;
-           (*env)[variable] = new (ctx.mem) Number(low->pstate(), ++i)) {
+           ++i) {
+        it->value(i);
+        env->set_local(variable, it);
         val = body->perform(this);
         if (val) break;
       }
@@ -108,15 +177,21 @@ namespace Sass {
       if (f->is_inclusive()) --end;
       for (double i = start;
            i > end;
-           (*env)[variable] = new (ctx.mem) Number(low->pstate(), --i)) {
+           --i) {
+        it->value(i);
+        env->set_local(variable, it);
         val = body->perform(this);
         if (val) break;
       }
     }
-    env = new_env.parent();
+    // restore original environment
+    if (!old_var) env->del_local(variable);
+    else env->set_local(variable, old_var);
     return val;
   }
 
+  // Eval does not create a new env scope
+  // But iteration vars are reset afterwards
   Expression* Eval::operator()(Each* e)
   {
     vector<string> variables(e->variables());
@@ -133,10 +208,12 @@ namespace Sass {
     else {
       list = static_cast<List*>(expr);
     }
-    Env new_env;
-    for (size_t i = 0, L = variables.size(); i < L; ++i) new_env[variables[i]] = 0;
-    new_env.link(env);
-    env = &new_env;
+    // remember variables and then reset them
+    vector<AST_Node*> old_vars(variables.size());
+    for (size_t i = 0, L = variables.size(); i < L; ++i) {
+      old_vars[i] = env->has_local(variables[i]) ? env->get_local(variables[i]) : 0;
+      env->set_local(variables[i], 0);
+    }
     Block* body = e->block();
     Expression* val = 0;
 
@@ -148,10 +225,10 @@ namespace Sass {
           List* variable = new (ctx.mem) List(map->pstate(), 2, List::SPACE);
           *variable << key;
           *variable << value;
-          (*env)[variables[0]] = variable;
+          env->set_local(variables[0], variable);
         } else {
-          (*env)[variables[0]] = key;
-          (*env)[variables[1]] = value;
+          env->set_local(variables[0], key);
+          env->set_local(variables[1], value);
         }
 
         val = body->perform(this);
@@ -170,10 +247,10 @@ namespace Sass {
         }
         for (size_t j = 0, K = variables.size(); j < K; ++j) {
           if (j < variable->length()) {
-            (*env)[variables[j]] = (*variable)[j];
+            env->set_local(variables[j], (*variable)[j]);
           }
           else {
-            (*env)[variables[j]] = new (ctx.mem) Null(expr->pstate());
+            env->set_local(variables[j], new (ctx.mem) Null(expr->pstate()));
           }
           val = body->perform(this);
           if (val) break;
@@ -181,7 +258,11 @@ namespace Sass {
         if (val) break;
       }
     }
-    env = new_env.parent();
+    // restore original environment
+    for (size_t j = 0, K = variables.size(); j < K; ++j) {
+      if(!old_vars[j]) env->del_local(variables[j]);
+      else env->set_local(variables[j], old_vars[j]);
+    }
     return val;
   }
 
@@ -204,7 +285,7 @@ namespace Sass {
   Expression* Eval::operator()(Warning* w)
   {
     Expression* message = w->message()->perform(this);
-    To_String to_string;
+    To_String to_string(&ctx);
 
     // try to use generic function
     if (env->has("@warn[f]")) {
@@ -212,12 +293,13 @@ namespace Sass {
       Definition* def = static_cast<Definition*>((*env)["@warn[f]"]);
       // Block*          body   = def->block();
       // Native_Function func   = def->native_function();
-      Sass_C_Function c_func = def->c_function();
+      Sass_Function_Entry c_function = def->c_function();
+      Sass_Function_Fn c_func = sass_function_get_function(c_function);
 
       To_C to_c;
       union Sass_Value* c_args = sass_make_list(1, SASS_COMMA);
       sass_list_set_value(c_args, 0, message->perform(&to_c));
-      Sass_Value* c_val = c_func(c_args, def->cookie());
+      Sass_Value* c_val = c_func(c_args, c_function, ctx.c_options);
       sass_delete_value(c_args);
       sass_delete_value(c_val);
       return 0;
@@ -235,7 +317,7 @@ namespace Sass {
   Expression* Eval::operator()(Error* e)
   {
     Expression* message = e->message()->perform(this);
-    To_String to_string;
+    To_String to_string(&ctx);
 
     // try to use generic function
     if (env->has("@error[f]")) {
@@ -243,12 +325,13 @@ namespace Sass {
       Definition* def = static_cast<Definition*>((*env)["@error[f]"]);
       // Block*          body   = def->block();
       // Native_Function func   = def->native_function();
-      Sass_C_Function c_func = def->c_function();
+      Sass_Function_Entry c_function = def->c_function();
+      Sass_Function_Fn c_func = sass_function_get_function(c_function);
 
       To_C to_c;
       union Sass_Value* c_args = sass_make_list(1, SASS_COMMA);
       sass_list_set_value(c_args, 0, message->perform(&to_c));
-      Sass_Value* c_val = c_func(c_args, def->cookie());
+      Sass_Value* c_val = c_func(c_args, c_function, ctx.c_options);
       sass_delete_value(c_args);
       sass_delete_value(c_val);
       return 0;
@@ -256,17 +339,14 @@ namespace Sass {
     }
 
     string result(unquote(message->perform(&to_string)));
-    Backtrace top(backtrace, e->pstate(), "");
-    cerr << "Error: " << result;
-    cerr << top.to_string(true);
-    cerr << endl << endl;
+    error(result, e->pstate());
     return 0;
   }
 
   Expression* Eval::operator()(Debug* d)
   {
     Expression* message = d->value()->perform(this);
-    To_String to_string;
+    To_String to_string(&ctx);
 
     // try to use generic function
     if (env->has("@debug[f]")) {
@@ -274,12 +354,13 @@ namespace Sass {
       Definition* def = static_cast<Definition*>((*env)["@debug[f]"]);
       // Block*          body   = def->block();
       // Native_Function func   = def->native_function();
-      Sass_C_Function c_func = def->c_function();
+      Sass_Function_Entry c_function = def->c_function();
+      Sass_Function_Fn c_func = sass_function_get_function(c_function);
 
       To_C to_c;
       union Sass_Value* c_args = sass_make_list(1, SASS_COMMA);
       sass_list_set_value(c_args, 0, message->perform(&to_c));
-      Sass_Value* c_val = c_func(c_args, def->cookie());
+      Sass_Value* c_val = c_func(c_args, c_function, ctx.c_options);
       sass_delete_value(c_args);
       sass_delete_value(c_val);
       return 0;
@@ -384,7 +465,18 @@ namespace Sass {
     if (l_type == Expression::COLOR && r_type == Expression::COLOR) {
       return op_colors(ctx, op_type, lhs, rhs);
     }
-    return op_strings(ctx, op_type, lhs, rhs);
+
+    Expression* ex = op_strings(ctx, op_type, lhs, rhs);
+    if (String_Constant* str = (String_Constant*) ex)
+    {
+      if (str->concrete_type() != Expression::STRING) return ex;
+      String_Constant* lstr = dynamic_cast<String_Constant*>(lhs);
+      String_Constant* rstr = dynamic_cast<String_Constant*>(rhs);
+      if (String_Constant* org = lstr ? lstr : rstr)
+      { str->quote_mark(org->quote_mark()); }
+    }
+    return ex;
+
   }
 
   Expression* Eval::operator()(Unary_Expression* u)
@@ -403,7 +495,7 @@ namespace Sass {
       return result;
     }
     else {
-      To_String to_string;
+      To_String to_string(&ctx);
       // Special cases: +/- variables which evaluate to null ouput just +/-,
       // but +/- null itself outputs the string
       if (operand->concrete_type() == Expression::NULL_VAL && typeid(*(u->operand())) == typeid(Variable)) {
@@ -420,7 +512,8 @@ namespace Sass {
 
   Expression* Eval::operator()(Function_Call* c)
   {
-    string full_name(c->name() + "[f]");
+    string name(Util::normalize_underscores(c->name()));
+    string full_name(name + "[f]");
     Arguments* args = c->arguments();
     if (full_name != "if[f]") {
       args = static_cast<Arguments*>(args->perform(this));
@@ -438,7 +531,7 @@ namespace Sass {
       Function_Call* lit = new (ctx.mem) Function_Call(c->pstate(),
                                                        c->name(),
                                                        args);
-      To_String to_string;
+      To_String to_string(&ctx);
       return new (ctx.mem) String_Constant(c->pstate(),
                                            lit->perform(&to_string));
     }
@@ -447,7 +540,7 @@ namespace Sass {
     Definition*     def    = static_cast<Definition*>((*env)[full_name]);
     Block*          body   = def->block();
     Native_Function func   = def->native_function();
-    Sass_C_Function c_func = def->c_function();
+    Sass_Function_Entry c_function = def->c_function();
 
     if (full_name != "if[f]") {
       for (size_t i = 0, L = args->length(); i < L; ++i) {
@@ -498,8 +591,9 @@ namespace Sass {
       env = old_env;
     }
     // else if it's a user-defined c function
-    else if (c_func) {
+    else if (c_function) {
 
+      Sass_Function_Fn c_func = sass_function_get_function(c_function);
       if (full_name == "*[f]") {
         String_Constant *str = new (ctx.mem) String_Constant(c->pstate(), c->name());
         Arguments* new_args = new (ctx.mem) Arguments(c->pstate());
@@ -517,14 +611,14 @@ namespace Sass {
       backtrace = &here;
 
       To_C to_c;
-      union Sass_Value* c_args = sass_make_list(env->current_frame().size(), SASS_COMMA);
+      union Sass_Value* c_args = sass_make_list(env->local_frame().size(), SASS_COMMA);
       for(size_t i = 0; i < params[0].length(); i++) {
         string key = params[0][i]->name();
-        AST_Node* node = env->current_frame().at(key);
+        AST_Node* node = env->local_frame().at(key);
         Expression* arg = static_cast<Expression*>(node);
         sass_list_set_value(c_args, i, arg->perform(&to_c));
       }
-      Sass_Value* c_val = c_func(c_args, def->cookie());
+      Sass_Value* c_val = c_func(c_args, c_function, ctx.c_options);
       if (sass_value_get_tag(c_val) == SASS_ERROR) {
         error("error in C function " + c->name() + ": " + sass_error_get_message(c_val), c->pstate(), backtrace);
       } else if (sass_value_get_tag(c_val) == SASS_WARNING) {
@@ -589,11 +683,11 @@ namespace Sass {
 
   Expression* Eval::operator()(Variable* v)
   {
-    To_String to_string;
+    To_String to_string(&ctx);
     string name(v->name());
     Expression* value = 0;
     if (env->has(name)) value = static_cast<Expression*>((*env)[name]);
-    else error("unbound variable " + v->name(), v->pstate());
+    else error("Undefined variable: \"" + v->name() + "\".", v->pstate());
     // cerr << "name: " << v->name() << "; type: " << typeid(*value).name() << "; value: " << value->perform(&to_string) << endl;
     if (typeid(*value) == typeid(Argument)) value = static_cast<Argument*>(value)->value();
 
@@ -603,7 +697,15 @@ namespace Sass {
       static_cast<Number*>(value)->zero(true);
     }
     else if (value->concrete_type() == Expression::STRING) {
-      value = new (ctx.mem) String_Constant(*static_cast<String_Constant*>(value));
+      if (auto str = dynamic_cast<String_Quoted*>(value)) {
+        value = new (ctx.mem) String_Quoted(*str);
+      } else if (auto str = dynamic_cast<String_Constant*>(value)) {
+        if (str->quote_mark()) {
+          value = new (ctx.mem) String_Quoted(str->pstate(), str->perform(&to_string));
+        } else {
+          value = new (ctx.mem) String_Constant(str->pstate(), unquote(str->value()));
+        }
+      }
     }
     else if (value->concrete_type() == Expression::LIST) {
       value = new (ctx.mem) List(*static_cast<List*>(value));
@@ -643,20 +745,20 @@ namespace Sass {
     {
       case Textual::NUMBER:
         result = new (ctx.mem) Number(t->pstate(),
-                                      atof(num.c_str()),
+                                      sass_atof(num.c_str()),
                                       "",
                                       zero);
         break;
       case Textual::PERCENTAGE:
         result = new (ctx.mem) Number(t->pstate(),
-                                      atof(num.c_str()),
+                                      sass_atof(num.c_str()),
                                       "%",
                                       zero);
         break;
       case Textual::DIMENSION:
         result = new (ctx.mem) Number(t->pstate(),
-                                      atof(num.c_str()),
-                                      Token(number(text.c_str()), t->pstate()),
+                                      sass_atof(num.c_str()),
+                                      Token(number(text.c_str())),
                                       zero);
         break;
       case Textual::HEX: {
@@ -711,28 +813,92 @@ namespace Sass {
     }
   }
 
+  string Eval::interpolation(Expression* s) {
+    if (String_Quoted* str_quoted = dynamic_cast<String_Quoted*>(s)) {
+      if (str_quoted->quote_mark()) {
+        if (str_quoted->quote_mark() == '*' || str_quoted->is_delayed()) {
+          return interpolation(new (ctx.mem) String_Constant(*str_quoted));
+        } else {
+          return string_escape(quote(str_quoted->value(), str_quoted->quote_mark()));
+        }
+      } else {
+        return evacuate_escapes(str_quoted->value());
+      }
+    } else if (String_Constant* str_constant = dynamic_cast<String_Constant*>(s)) {
+      string str = str_constant->value();
+      if (!str_constant->quote_mark()) str = unquote(str);
+      return evacuate_escapes(str);
+    } else if (String_Schema* str_schema = dynamic_cast<String_Schema*>(s)) {
+      string res = "";
+      for(auto i : str_schema->elements())
+        res += (interpolation(i));
+      //ToDo: do this in one step
+      auto esc = evacuate_escapes(res);
+      auto unq = unquote(esc);
+      if (unq == esc) {
+        return string_to_output(res);
+      } else {
+        return evacuate_quotes(unq);
+      }
+    } else if (List* list = dynamic_cast<List*>(s)) {
+      string acc = ""; // ToDo: different output styles
+      string sep = list->separator() == List::Separator::COMMA ? "," : " ";
+      if (ctx.output_style != COMPRESSED && sep == ",") sep += " ";
+      bool initial = false;
+      for(auto item : list->elements()) {
+        if (initial) acc += sep;
+        acc += interpolation(item);
+        initial = true;
+      }
+      return evacuate_quotes(acc);
+    } else if (Variable* var = dynamic_cast<Variable*>(s)) {
+      string name(var->name());
+      if (!env->has(name)) error("Undefined variable: \"" + var->name() + "\".", var->pstate());
+      Expression* value = static_cast<Expression*>((*env)[name]);
+      return evacuate_quotes(interpolation(value));
+    } else if (Binary_Expression* var = dynamic_cast<Binary_Expression*>(s)) {
+      Expression* ex = var->perform(this);
+      return evacuate_quotes(interpolation(ex));
+    } else if (Function_Call* var = dynamic_cast<Function_Call*>(s)) {
+      Expression* ex = var->perform(this);
+      return evacuate_quotes(interpolation(ex));
+    } else if (Parent_Selector* var = dynamic_cast<Parent_Selector*>(s)) {
+      Expression* ex = var->perform(this);
+      return evacuate_quotes(interpolation(ex));
+    } else if (Unary_Expression* var = dynamic_cast<Unary_Expression*>(s)) {
+      Expression* ex = var->perform(this);
+      return evacuate_quotes(interpolation(ex));
+    } else if (Selector* var = dynamic_cast<Selector*>(s)) {
+      Expression* ex = var->perform(this);
+      return evacuate_quotes(interpolation(ex));
+    } else {
+      To_String to_string(&ctx);
+      // to_string.in_decl_list = true;
+      return evacuate_quotes(s->perform(&to_string));
+    }
+  }
+
   Expression* Eval::operator()(String_Schema* s)
   {
     string acc;
-    // ctx._skip_source_map_update = true;
-    To_String to_string(&ctx);
-    // ctx._skip_source_map_update = false;
     for (size_t i = 0, L = s->length(); i < L; ++i) {
-      string chunk((*s)[i]->perform(this)->perform(&to_string));
-      if (((s->quote_mark() && is_quoted(chunk)) || !s->quote_mark()) && (*s)[i]->is_interpolant()) { // some redundancy in that test
-        acc += unquote(chunk);
-      }
-      else {
-        acc += chunk;
+      if (String_Quoted* str_quoted = dynamic_cast<String_Quoted*>((*s)[i])) {
+        if (!str_quoted->is_delayed()) str_quoted->value(string_eval_escapes(str_quoted->value()));
       }
+      acc += interpolation((*s)[i]);
     }
-    return new (ctx.mem) String_Constant(s->pstate(),
-                                         acc);
+    String_Quoted* str = new (ctx.mem) String_Quoted(s->pstate(), acc);
+    if (!str->quote_mark()) {
+      str->value(string_unescape(str->value()));
+    } else if (str->quote_mark()) {
+      str->quote_mark('*');
+    }
+    return str;
   }
 
   Expression* Eval::operator()(String_Constant* s)
   {
-    if (!s->is_delayed() && ctx.names_to_colors.count(s->value())) {
+    if (!s->quote_mark() && !s->is_delayed() && ctx.names_to_colors.count(s->value())) {
       Color* c = new (ctx.mem) Color(*ctx.names_to_colors[s->value()]);
       c->pstate(s->pstate());
       c->disp(s->value());
@@ -783,7 +949,7 @@ namespace Sass {
 
   Expression* Eval::operator()(Media_Query* q)
   {
-    To_String to_string;
+    To_String to_string(&ctx);
     String* t = q->media_type();
     t = static_cast<String*>(t ? t->perform(this) : 0);
     Media_Query* qq = new (ctx.mem) Media_Query(q->pstate(),
@@ -860,6 +1026,15 @@ namespace Sass {
     return 0;
   }
 
+  Expression* Eval::operator()(Parent_Selector* p)
+  {
+    Selector* s = p->perform(contextualize);
+    // access to parent selector may return 0
+    Selector_List* l = static_cast<Selector_List*>(s);
+    if (!s) { l = new (ctx.mem) Selector_List(p->pstate()); }
+    return l->perform(listize);
+  }
+
   inline Expression* Eval::fallback_impl(AST_Node* n)
   {
     return static_cast<Expression*>(n);
@@ -1019,8 +1194,9 @@ namespace Sass {
       case Binary_Expression::SUB:
       case Binary_Expression::DIV: {
         string sep(op == Binary_Expression::SUB ? "-" : "/");
-        To_String to_string;
-        string color(r->sixtuplet() ? r->perform(&to_string) :
+        To_String to_string(&ctx);
+        string color(r->sixtuplet() && (ctx.output_style != COMPRESSED) ?
+                     r->perform(&to_string) :
                      Util::normalize_sixtuplet(r->perform(&to_string)));
         return new (ctx.mem) String_Constant(l->pstate(),
                                              l->perform(&to_string)
@@ -1069,20 +1245,18 @@ namespace Sass {
 
   Expression* op_strings(Context& ctx, Binary_Expression::Type op, Expression* lhs, Expression*rhs)
   {
-    To_String to_string;
+    To_String to_string(&ctx);
     Expression::Concrete_Type ltype = lhs->concrete_type();
     Expression::Concrete_Type rtype = rhs->concrete_type();
 
     string lstr(lhs->perform(&to_string));
     string rstr(rhs->perform(&to_string));
 
-    bool l_str_quoted = ((Sass::String*)lhs) && ((Sass::String*)lhs)->needs_unquoting();
-    bool r_str_quoted = ((Sass::String*)rhs) && ((Sass::String*)rhs)->needs_unquoting();
+    bool l_str_quoted = ((Sass::String*)lhs) && ((Sass::String*)lhs)->sass_fix_1291();
+    bool r_str_quoted = ((Sass::String*)rhs) && ((Sass::String*)rhs)->sass_fix_1291();
     bool l_str_color = ltype == Expression::STRING && ctx.names_to_colors.count(lstr) && !l_str_quoted;
     bool r_str_color = rtype == Expression::STRING && ctx.names_to_colors.count(rstr) && !r_str_quoted;
 
-    bool unquoted = false;
-    if (ltype == Expression::STRING && lstr[0] != '"' && lstr[0] != '\'') unquoted = true;
     if (l_str_color && r_str_color) {
       return op_colors(ctx, op, ctx.names_to_colors[lstr], ctx.names_to_colors[rstr]);
     }
@@ -1108,12 +1282,10 @@ namespace Sass {
     }
     if (ltype == Expression::NULL_VAL) error("invalid null operation: \"null plus "+quote(unquote(rstr), '"')+"\".", lhs->pstate());
     if (rtype == Expression::NULL_VAL) error("invalid null operation: \""+quote(unquote(lstr), '"')+" plus null\".", lhs->pstate());
-    char q = '\0';
-    if (lstr[0] == '"' || lstr[0] == '\'') q = lstr[0];
-    else if (rstr[0] == '"' || rstr[0] == '\'') q = rstr[0];
-    string result(unquote(lstr) + sep + unquote(rstr));
-    return new (ctx.mem) String_Constant(lhs->pstate(),
-                               unquoted ? result : quote(result, q));
+    string result((lstr) + sep + (rstr));
+    String_Quoted* str = new (ctx.mem) String_Quoted(lhs->pstate(), result);
+    str->quote_mark(0);
+    return str;
   }
 
   Expression* cval_to_astnode(Sass_Value* v, Context& ctx, Backtrace* backtrace, ParserState pstate)
