@@ -4,7 +4,7 @@
 #include "expand.hpp"
 #include "bind.hpp"
 #include "eval.hpp"
-#include "contextualize.hpp"
+#include "contextualize_eval.hpp"
 #include "to_string.hpp"
 #include "backtrace.hpp"
 #include "context.hpp"
@@ -12,10 +12,10 @@
 
 namespace Sass {
 
-  Expand::Expand(Context& ctx, Eval* eval, Contextualize* contextualize, Env* env, Backtrace* bt)
+  Expand::Expand(Context& ctx, Eval* eval, Contextualize_Eval* contextualize_eval, Env* env, Backtrace* bt)
   : ctx(ctx),
     eval(eval),
-    contextualize(contextualize),
+    contextualize_eval(contextualize_eval),
     env(env),
     block_stack(vector<Block*>()),
     property_stack(vector<String*>()),
@@ -47,43 +47,43 @@ namespace Sass {
     if (in_keyframes) {
       To_String to_string;
       Keyframe_Rule* k = new (ctx.mem) Keyframe_Rule(r->pstate(), r->block()->perform(this)->block());
-      if (r->selector()) {
-        string s(r->selector()->perform(eval->with(env, backtrace))->perform(&to_string));
-        String_Constant* ss = new (ctx.mem) String_Constant(r->selector()->pstate(), s);
-        k->rules(ss);
-      }
+      if (r->selector()) k->selector(r->selector()->perform(contextualize_eval->with(0, env, backtrace)));
       in_at_root = old_in_at_root;
       old_in_at_root = false;
       return k;
     }
 
-    Contextualize* contextual = contextualize->with(selector_stack.back(), env, backtrace);
+    Contextualize_Eval* contextual = contextualize_eval->with(selector_stack.back(), env, backtrace);
     if (old_in_at_root && !r->selector()->has_reference())
-      contextual = contextualize->with(at_root_selector_stack.back(), env, backtrace);
+      contextual = contextualize_eval->with(at_root_selector_stack.back(), env, backtrace);
 
     Selector* sel_ctx = r->selector()->perform(contextual);
+    if (sel_ctx == 0) throw "Cannot expand null selector";
 
-    Inspect isp(0);
+    Emitter emitter(&ctx);
+    Inspect isp(emitter);
     sel_ctx->perform(&isp);
     string str = isp.get_buffer();
     str += ";";
 
-    Parser p(ctx, ParserState("[REPARSE]", 0));
+    Parser p(ctx, r->pstate());
+    p.block_stack.push_back(r->selector() ? r->selector()->last_block() : 0);
+    p.last_media_block = r->selector() ? r->selector()->media_block() : 0;
     p.source   = str.c_str();
     p.position = str.c_str();
     p.end      = str.c_str() + strlen(str.c_str());
     Selector_List* sel_lst = p.parse_selector_group();
-    sel_lst->pstate(isp.source_map.remap(sel_lst->pstate()));
+    // sel_lst->pstate(isp.remap(sel_lst->pstate()));
 
     for(size_t i = 0; i < sel_lst->length(); i++) {
 
       Complex_Selector* pIter = (*sel_lst)[i];
       while (pIter) {
         Compound_Selector* pHead = pIter->head();
-        pIter->pstate(isp.source_map.remap(pIter->pstate()));
+        // pIter->pstate(isp.remap(pIter->pstate()));
         if (pHead) {
-          pHead->pstate(isp.source_map.remap(pHead->pstate()));
-          (*pHead)[0]->pstate(isp.source_map.remap((*pHead)[0]->pstate()));
+          // pHead->pstate(isp.remap(pHead->pstate()));
+          // (*pHead)[0]->pstate(isp.remap((*pHead)[0]->pstate()));
         }
         pIter = pIter->tail();
       }
@@ -91,9 +91,11 @@ namespace Sass {
     sel_ctx = sel_lst;
 
     selector_stack.push_back(sel_ctx);
+    Block* blk = r->block()->perform(this)->block();
     Ruleset* rr = new (ctx.mem) Ruleset(r->pstate(),
                                         sel_ctx,
-                                        r->block()->perform(this)->block());
+                                        blk);
+    rr->tabs(r->tabs());
     selector_stack.pop_back();
     in_at_root = old_in_at_root;
     old_in_at_root = false;
@@ -122,6 +124,9 @@ namespace Sass {
         dec->property(combined_prop);
         *current_block << dec;
       }
+      else if (typeid(*stm) == typeid(Comment)) {
+        // drop comments in propsets
+      }
       else {
         error("contents of namespaced properties must result in style declarations only", stm->pstate(), backtrace);
       }
@@ -144,13 +149,14 @@ namespace Sass {
 
   Statement* Expand::operator()(Media_Block* m)
   {
-    To_String to_string;
+    To_String to_string(&ctx);
     Expression* mq = m->media_queries()->perform(eval->with(env, backtrace));
     mq = Parser::from_c_str(mq->perform(&to_string).c_str(), ctx, mq->pstate()).parse_media_queries();
     Media_Block* mm = new (ctx.mem) Media_Block(m->pstate(),
                                                 static_cast<List*>(mq),
                                                 m->block()->perform(this)->block(),
                                                 selector_stack.back());
+    mm->tabs(m->tabs());
     return mm;
   }
 
@@ -178,7 +184,7 @@ namespace Sass {
     Block* ab = a->block();
     Selector* as = a->selector();
     Expression* av = a->value();
-    if (as) as = as->perform(contextualize->with(0, env, backtrace));
+    if (as) as = as->perform(contextualize_eval->with(0, env, backtrace));
     else if (av) av = av->perform(eval->with(env, backtrace));
     Block* bb = ab ? ab->perform(this)->block() : 0;
     At_Rule* aa = new (ctx.mem) At_Rule(a->pstate(),
@@ -194,25 +200,74 @@ namespace Sass {
   {
     String* old_p = d->property();
     String* new_p = static_cast<String*>(old_p->perform(eval->with(env, backtrace)));
-    Expression* value = d->value()->perform(eval->with(env, backtrace));
-
+    Selector* p = selector_stack.size() <= 1 ? 0 : selector_stack.back();
+    Expression* value = d->value()->perform(eval->with(p, env, backtrace));
     if (value->is_invisible() && !d->is_important()) return 0;
-
-    return new (ctx.mem) Declaration(d->pstate(),
-                                     new_p,
-                                     value,
-                                     d->is_important());
+    Declaration* decl = new (ctx.mem) Declaration(d->pstate(),
+                                                  new_p,
+                                                  value,
+                                                  d->is_important());
+    decl->tabs(d->tabs());
+    return decl;
   }
 
   Statement* Expand::operator()(Assignment* a)
   {
     string var(a->variable());
-    if (env->has(var)) {
-      Expression* v = static_cast<Expression*>((*env)[var]);
-      if (!a->is_guarded() || v->concrete_type() == Expression::NULL_VAL) (*env)[var] = a->value()->perform(eval->with(env, backtrace));
+    Selector* p = selector_stack.size() <= 1 ? 0 : selector_stack.back();
+    if (a->is_global()) {
+      if (a->is_default()) {
+        if (env->has_global(var)) {
+          Expression* e = dynamic_cast<Expression*>(env->get_global(var));
+          if (!e || e->concrete_type() == Expression::NULL_VAL) {
+            env->set_global(var, a->value()->perform(eval->with(p, env, backtrace)));
+          }
+        }
+        else {
+          env->set_global(var, a->value()->perform(eval->with(p, env, backtrace)));
+        }
+      }
+      else {
+        env->set_global(var, a->value()->perform(eval->with(p, env, backtrace)));
+      }
+    }
+    else if (a->is_default()) {
+      if (env->has_lexical(var)) {
+        auto cur = env;
+        while (cur && cur->is_lexical()) {
+          if (cur->has_local(var)) {
+            if (AST_Node* node = cur->get_local(var)) {
+              Expression* e = dynamic_cast<Expression*>(node);
+              if (!e || e->concrete_type() == Expression::NULL_VAL) {
+                cur->set_local(var, a->value()->perform(eval->with(p, env, backtrace)));
+              }
+            }
+            else {
+              throw runtime_error("Env not in sync");
+            }
+            return 0;
+          }
+          cur = cur->parent();
+        }
+        throw runtime_error("Env not in sync");
+      }
+      else if (env->has_global(var)) {
+        if (AST_Node* node = env->get_global(var)) {
+          Expression* e = dynamic_cast<Expression*>(node);
+          if (!e || e->concrete_type() == Expression::NULL_VAL) {
+            env->set_global(var, a->value()->perform(eval->with(p, env, backtrace)));
+          }
+        }
+      }
+      else if (env->is_lexical()) {
+        env->set_local(var, a->value()->perform(eval->with(p, env, backtrace)));
+      }
+      else {
+        env->set_local(var, a->value()->perform(eval->with(p, env, backtrace)));
+      }
     }
     else {
-      env->current_frame()[var] = a->value()->perform(eval->with(env, backtrace));
+      env->set_lexical(var, a->value()->perform(eval->with(p, env, backtrace)));
     }
     return 0;
   }
@@ -256,7 +311,7 @@ namespace Sass {
   Statement* Expand::operator()(Comment* c)
   {
     // TODO: eval the text, once we're parsing/storing it as a String_Schema
-    return new (ctx.mem) Comment(c->pstate(), static_cast<String*>(c->text()->perform(eval->with(env, backtrace))));
+    return new (ctx.mem) Comment(c->pstate(), static_cast<String*>(c->text()->perform(eval->with(env, backtrace))), c->is_important());
   }
 
   Statement* Expand::operator()(If* i)
@@ -271,6 +326,8 @@ namespace Sass {
     return 0;
   }
 
+  // For does not create a new env scope
+  // But iteration vars are reset afterwards
   Statement* Expand::operator()(For* f)
   {
     string variable(f->variable());
@@ -282,32 +339,49 @@ namespace Sass {
     if (high->concrete_type() != Expression::NUMBER) {
       error("upper bound of `@for` directive must be numeric", high->pstate(), backtrace);
     }
-    double start = static_cast<Number*>(low)->value();
-    double end = static_cast<Number*>(high)->value();
-    Env new_env;
-    new_env[variable] = new (ctx.mem) Number(low->pstate(), start);
-    new_env.link(env);
-    env = &new_env;
+    Number* sass_start = static_cast<Number*>(low);
+    Number* sass_end = static_cast<Number*>(high);
+    // check if units are valid for sequence
+    if (sass_start->unit() != sass_end->unit()) {
+      stringstream msg; msg << "Incompatible units: '"
+        << sass_start->unit() << "' and '"
+        << sass_end->unit() << "'.";
+      error(msg.str(), low->pstate(), backtrace);
+    }
+    double start = sass_start->value();
+    double end = sass_end->value();
+    // only create iterator once in this environment
+    Number* it = new (env->mem) Number(low->pstate(), start, sass_end->unit());
+    AST_Node* old_var = env->has_local(variable) ? env->get_local(variable) : 0;
+    env->set_local(variable, it);
     Block* body = f->block();
     if (start < end) {
       if (f->is_inclusive()) ++end;
       for (double i = start;
            i < end;
-           (*env)[variable] = new (ctx.mem) Number(low->pstate(), ++i)) {
+           ++i) {
+        it->value(i);
+        env->set_local(variable, it);
         append_block(body);
       }
     } else {
       if (f->is_inclusive()) --end;
       for (double i = start;
            i > end;
-           (*env)[variable] = new (ctx.mem) Number(low->pstate(), --i)) {
+           --i) {
+        it->value(i);
+        env->set_local(variable, it);
         append_block(body);
       }
     }
-    env = new_env.parent();
+    // restore original environment
+    if (!old_var) env->del_local(variable);
+    else env->set_local(variable, old_var);
     return 0;
   }
 
+  // Eval does not create a new env scope
+  // But iteration vars are reset afterwards
   Statement* Expand::operator()(Each* e)
   {
     vector<string> variables(e->variables());
@@ -324,10 +398,12 @@ namespace Sass {
     else {
       list = static_cast<List*>(expr);
     }
-    Env new_env;
-    for (size_t i = 0, L = variables.size(); i < L; ++i) new_env[variables[i]] = 0;
-    new_env.link(env);
-    env = &new_env;
+    // remember variables and then reset them
+    vector<AST_Node*> old_vars(variables.size());
+    for (size_t i = 0, L = variables.size(); i < L; ++i) {
+      old_vars[i] = env->has_local(variables[i]) ? env->get_local(variables[i]) : 0;
+      env->set_local(variables[i], 0);
+    }
     Block* body = e->block();
 
     if (map) {
@@ -339,10 +415,10 @@ namespace Sass {
           List* variable = new (ctx.mem) List(map->pstate(), 2, List::SPACE);
           *variable << k;
           *variable << v;
-          (*env)[variables[0]] = variable;
+          env->set_local(variables[0], variable);
         } else {
-          (*env)[variables[0]] = k;
-          (*env)[variables[1]] = v;
+          env->set_local(variables[0], k);
+          env->set_local(variables[1], v);
         }
         append_block(body);
       }
@@ -359,16 +435,20 @@ namespace Sass {
         }
         for (size_t j = 0, K = variables.size(); j < K; ++j) {
           if (j < variable->length()) {
-            (*env)[variables[j]] = (*variable)[j]->perform(eval->with(env, backtrace));
+            env->set_local(variables[j], (*variable)[j]->perform(eval->with(env, backtrace)));
           }
           else {
-            (*env)[variables[j]] = new (ctx.mem) Null(expr->pstate());
+            env->set_local(variables[j], new (ctx.mem) Null(expr->pstate()));
           }
         }
         append_block(body);
       }
     }
-    env = new_env.parent();
+    // restore original environment
+    for (size_t j = 0, K = variables.size(); j < K; ++j) {
+      if(!old_vars[j]) env->del_local(variables[j]);
+      else env->set_local(variables[j], old_vars[j]);
+    }
     return 0;
   }
 
@@ -390,28 +470,32 @@ namespace Sass {
 
   Statement* Expand::operator()(Extension* e)
   {
-    To_String to_string;
+    To_String to_string(&ctx);
     Selector_List* extender = static_cast<Selector_List*>(selector_stack.back());
     if (!extender) return 0;
-    Selector_List* extendee = static_cast<Selector_List*>(e->selector()->perform(contextualize->with(0, env, backtrace)));
-    if (extendee->length() != 1) {
-      error("selector groups may not be extended", extendee->pstate(), backtrace);
-    }
-    Complex_Selector* c = (*extendee)[0];
-    if (!c->head() || c->tail()) {
-      error("nested selectors may not be extended", c->pstate(), backtrace);
-    }
-    Compound_Selector* s = c->head();
-
-    // // need to convert the compound selector into a by-value data structure
-    // vector<string> target_vec;
-    // for (size_t i = 0, L = s->length(); i < L; ++i)
-    // { target_vec.push_back((*s)[i]->perform(&to_string)); }
-
-    for (size_t i = 0, L = extender->length(); i < L; ++i) {
-      // let's test this out
-      // cerr << "REGISTERING EXTENSION REQUEST: " << (*extender)[i]->perform(&to_string) << " <- " << s->perform(&to_string) << endl;
-      ctx.subset_map.put(s->to_str_vec(), make_pair((*extender)[i], s));
+    Contextualize_Eval* eval = contextualize_eval->with(0, env, backtrace);
+    Selector_List* selector_list = static_cast<Selector_List*>(e->selector());
+    Selector_List* contextualized = static_cast<Selector_List*>(selector_list->perform(eval));
+    // ToDo: remove once feature proves stable!
+    // if (contextualized->length() != 1) {
+    //   error("selector groups may not be extended", extendee->pstate(), backtrace);
+    // }
+    for (auto complex_sel : contextualized->elements()) {
+      Complex_Selector* c = complex_sel;
+      if (!c->head() || c->tail()) {
+        error("nested selectors may not be extended", c->pstate(), backtrace);
+      }
+      Compound_Selector* compound_sel = c->head();
+      compound_sel->is_optional(selector_list->is_optional());
+      // // need to convert the compound selector into a by-value data structure
+      // vector<string> target_vec;
+      // for (size_t i = 0, L = compound_sel->length(); i < L; ++i)
+      // { target_vec.push_back((*compound_sel)[i]->perform(&to_string)); }
+      for (size_t i = 0, L = extender->length(); i < L; ++i) {
+        // let's test this out
+        // cerr << "REGISTERING EXTENSION REQUEST: " << (*extender)[i]->perform(&to_string) << " <- " << compound_sel->perform(&to_string) << endl;
+        ctx.subset_map.put(compound_sel->to_str_vec(), make_pair((*extender)[i], compound_sel));
+      }
     }
     return 0;
   }
@@ -419,7 +503,7 @@ namespace Sass {
   Statement* Expand::operator()(Definition* d)
   {
     Definition* dd = new (ctx.mem) Definition(*d);
-    env->current_frame()[d->name() +
+    env->local_frame()[d->name() +
                         (d->type() == Definition::MIXIN ? "[m]" : "[f]")] = dd;
     // set the static link so we can have lexical scoping
     dd->environment(env);
@@ -435,8 +519,10 @@ namespace Sass {
     Definition* def = static_cast<Definition*>((*env)[full_name]);
     Block* body = def->block();
     Parameters* params = def->parameters();
+    Selector* p = selector_stack.size() <= 1 ? 0 : selector_stack.back();
+
     Arguments* args = static_cast<Arguments*>(c->arguments()
-                                               ->perform(eval->with(env, backtrace)));
+                                               ->perform(eval->with(p, env, backtrace)));
     Backtrace here(backtrace, c->pstate(), ", in mixin `" + c->name() + "`");
     backtrace = &here;
     Env new_env;
@@ -447,9 +533,10 @@ namespace Sass {
                                                    "@content",
                                                    new (ctx.mem) Parameters(c->pstate()),
                                                    c->block(),
+                                                   &ctx,
                                                    Definition::MIXIN);
       thunk->environment(env);
-      new_env.current_frame()["@content[m]"] = thunk;
+      new_env.local_frame()["@content[m]"] = thunk;
     }
     bind("mixin " + c->name(), params, args, ctx, &new_env, eval);
     Env* old_env = env;
