@@ -6,15 +6,17 @@
 #include <iostream>
 
 #include "ast.hpp"
+#include "eval.hpp"
 #include "operation.hpp"
 #include "environment.hpp"
+#include "contextualize.hpp"
 
 namespace Sass {
   using namespace std;
 
-  struct Context;
+  class Context;
   class Eval;
-  class Contextualize;
+  class Contextualize_Eval;
   typedef Environment<AST_Node*> Env;
   struct Backtrace;
 
@@ -22,7 +24,7 @@ namespace Sass {
 
     Context&          ctx;
     Eval*             eval;
-    Contextualize*    contextualize;
+    Contextualize_Eval*    contextualize_eval;
     Env*              env;
     vector<Block*>    block_stack;
     vector<String*>   property_stack;
@@ -35,7 +37,7 @@ namespace Sass {
     Statement* fallback_impl(AST_Node* n);
 
   public:
-    Expand(Context&, Eval*, Contextualize*, Env*, Backtrace*);
+    Expand(Context&, Eval*, Contextualize_Eval*, Env*, Backtrace*);
     virtual ~Expand() { }
 
     using Operation<Statement*>::operator();
