@@ -38,7 +38,7 @@ namespace Sass {
     end
 	*/
   Node paths(const Node& arrs, Context& ctx) {
-    To_String to_string;
+    To_String to_string(&ctx);
 
     Node loopStart = Node::createCollection();
     loopStart.collection()->push_back(Node::createCollection());
